import click
from .api_wrapper import request_no_response
from .api_wrapper import request_data, tenb_connection

tio = tenb_connection()


@click.group(help="Delete objects from Tenable IO")
def delete():
    pass


@delete.command(help="Delete assets by Tag: tag_category:tag_value - Example - OS:Linux")
@click.argument('tag_string')
def bytag(tag_string):
    tag_tuple = tag_string.split(':')
    cat = tag_tuple[0]
    val = tag_tuple[1]
    if bytag != '':
        click.echo("\nI'm deleting all of the assets associated with your Tag\n")
        payload = {'query': {'field': "tag.{}".format(cat), 'operator': 'set-has', 'value': str(val)}}
        request_data('POST', '/api/v2/assets/bulk-jobs/delete', payload=payload)


@delete.command(help='Delete a Scan by Scan ID')
@click.argument('tid')
def scan(tid):
    click.echo("\nI'm deleting your Scan Now")
    tio.scans.delete(str(tid))


@delete.command(help='Delete an access group by UUID')
@click.argument('tid')
def agroup(tid):
    click.echo("\nI'm deleting your Access Group Now")
    tio.access_groups.delete(str(tid))


@delete.command(help='Delete a target-group by target-group ID')
@click.argument('tid')
def tgroup(tid):
    click.echo("\nI'm deleting your Target group Now")
    tio.target_groups.delete(str(tid))


@delete.command(help='Delete a Policy by Policy ID')
@click.argument('tid')
def policy(tid):
    click.echo("\nI'm deleting your Policy Now")
    tio.policies.delete(str(tid))


@delete.command(help='Delete an Asset by Asset UUID')
@click.argument('tid')
def asset(tid):
    click.echo("\nI'm deleting your asset Now")
    tio.assets.delete(str(tid))


@delete.command(help='Delete a container by \'/repository/image/tag\'')
@click.argument('tid')
def container(tid):
    click.echo("\nI'm deleting your container")
    request_no_response('DELETE', '/container-security/api/v2/images/' + str(tid))


@delete.command(help='Delete Tag Value by Value UUID')
@click.argument('tid')
def value(tid):
    click.echo("\nI'm deleting your Tag Value")
    tio.tags.delete(str(tid))


@delete.command(help='Delete Tag Category by Category UUID')
@click.argument('tid')
def category(tid):
    click.echo("\nI'm Deleting your Category")
    tio.tags.delete_category(str(tid))


@delete.command(help='Delete repository from Container Security')
@click.argument('tid')
def repository(tid):
    click.echo("\nI'm Deleting your Repository")
    request_no_response('delete', '/container-security/api/v2/' + str(tid))


@delete.command(help='Delete a user by User ID - Not UUID')
@click.argument('tid')
def user(tid):
    click.echo("\nI'm Deleting the User you requested")
    tio.users.delete(str(tid))


@delete.command(help='Delete a user group by the Group ID')
@click.argument('tid')
def usergroup(tid):
    click.echo("\nI'm Deleting the User you requested")
    tio.groups.delete(str(tid))


@delete.command(help='Delete a tag by Category/Value pair')
@click.option('--c', default='', required=True, help="Category to delete")
@click.option('--v', default='', required=True, help="Value to Delete")
def tag(c,v):
    tagdata = request_data('GET', '/tags/values')
    for tags in tagdata['values']:
        if c == tags['category_name']:
            if v == tags['value']:
                value_uuid = tags['uuid']
                request_no_response('DELETE', '/tags/values/' + str(value_uuid))



import csv
from .api_wrapper import request_data


def was_detailed_export():
    # Crete a csv file object
    with open('was_detailed_data.csv', mode='w') as csv_file:
        agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')
        # write our Header information first
        header_list = ["Scan Name", "APP", "Risk", "Plugin ID", "Plugin Name", "Description", "Solution", "See Also",
                       "CVSS", "CVSS3", "CVEs", "CWE", "WASC", "OWASP", "Scan Finalized Date"]
        agent_writer.writerow(header_list)

        params = {"size": "1000"}
        # Grab all of the Scans
        data = request_data('GET', '/was/v2/scans', params=params)

        for scan_data in data['data']:
            was_scan_id = scan_data['scan_id']
            status = scan_data['status']

            # Ignore all scans that have not completed
            if status == 'completed':
                report = request_data('GET', '/was/v2/scans/' + was_scan_id + '/report')

                try:
                    name = report['config']['name']
                    finalized = report['scan']['finalized_at']
                    try:
                        target = report['scan']['target']
                    except KeyError:
                        target = report['config']['settings']['target']

                    for finding in report['findings']:
                        csv_list = []
                        owasp_list = []
                        description = finding['description']
                        try:
                            solution = finding['solution']
                        except KeyError:
                            solution = ''
                        try:
                            see_also = finding['see_also']
                        except KeyError:
                            see_also = ''

                        risk = finding['risk_factor']

                        try:
                            cvss = finding['cvss']
                        except KeyError:
                            cvss = ''
                        try:
                            cvss3 = finding['cvssv3']
                        except KeyError:
                            cvss3 = ''

                        cves = finding['cves']
                        try:
                            cwe = finding['cwe']
                        except KeyError:
                            cwe = ''

                        try:
                            wasc = finding['wasc']
                        except KeyError:
                            wasc = ''

                        # Grab multiples values here
                        for owasp in finding['owasp']:
                            if owasp['year'] == '2017':
                                owasp_list.append(owasp['category'])

                        # ignore info vulns
                        if risk != "info":
                            plugin_id = finding['plugin_id']
                            plugin_name = finding['name']

                            csv_list.append(name)
                            csv_list.append(target)
                            csv_list.append(risk)
                            csv_list.append(plugin_id)
                            csv_list.append(plugin_name)
                            csv_list.append(description)
                            csv_list.append(solution)
                            csv_list.append(see_also)
                            csv_list.append(cvss)
                            csv_list.append(cvss3)
                            csv_list.append(cves)
                            csv_list.append(cwe)
                            csv_list.append(wasc)
                            csv_list.append(owasp_list)
                            csv_list.append(finalized)

                            agent_writer.writerow(csv_list)
                except TypeError:
                    pass
        print()



from .database import new_db_connection, create_table


def create_keys_table():
    database = r"navi.db"
    key_conn = new_db_connection(database)
    key_table = """CREATE TABLE IF NOT EXISTS keys (
                            access_key text,
                            secret_key text
                            );"""
    create_table(key_conn, key_table)


def create_diff_table():
    database = r"navi.db"
    diff_conn = new_db_connection(database)
    diff_table = """CREATE TABLE IF NOT EXISTS diff (
                        update_id integer PRIMARY KEY,
                        timestamp text,
                        days text,
                        update_type text,
                        exid text);"""
    create_table(diff_conn, diff_table)


def create_vulns_table():
    database = r"navi.db"
    vuln_conn = new_db_connection(database)
    vuln_table = """CREATE TABLE IF NOT EXISTS vulns (
                            asset_ip text, 
                            asset_uuid text, 
                            asset_hostname text, 
                            first_found text, 
                            last_found text, 
                            output text, 
                            plugin_id text, 
                            plugin_name text, 
                            plugin_family text, 
                            port text, 
                            protocol text, 
                            severity text, 
                            scan_completed text, 
                            scan_started text, 
                            scan_uuid text, 
                            schedule_id text, 
                            state text,
                            cves text,
                            score text,
                            exploit text
                            );"""
    vuln_conn.execute('pragma journal_mode=wal;')
    create_table(vuln_conn, vuln_table)


def create_assets_table():
    database = r"navi.db"
    asset_conn = new_db_connection(database)
    create_asset_table = """CREATE TABLE IF NOT EXISTS assets (
                            ip_address text,
                            hostname text,
                            fqdn text,
                            uuid text PRIMARY KEY,
                            first_found text,
                            last_found text, 
                            operating_system text,
                            mac_address text, 
                            agent_uuid text,
                            last_licensed_scan_date text,
                            network text,
                            acr text,
                            aes text,
                            aws_id text
                            );"""
    asset_conn.execute('pragma journal_mode=wal;')
    create_table(asset_conn, create_asset_table)


def create_tag_table():
    database = r"navi.db"
    tag_conn = new_db_connection(database)
    create_tags_table = """CREATE TABLE IF NOT EXISTS tags (
                        tag_id integer PRIMARY KEY,
                        asset_uuid text,
                        asset_ip,
                        tag_key text,
                        tag_uuid text,
                        tag_value text,
                        tag_added_date text
                        );"""
    tag_conn.execute('pragma journal_mode=wal;')
    create_table(tag_conn, create_tags_table)


def create_apps_table():
    database = r"navi.db"
    app_conn = new_db_connection(database)
    create_apps = """CREATE TABLE IF NOT EXISTS apps (
                            name text,
                            uuid text PRIMARY KEY, 
                            target text, 
                            scan_completed_time text,
                            pages_audited text,
                            pages_crawled text,
                            requests_made text, 
                            critical_count text,
                            high_count text,
                            medium_count text,
                            low_count text, 
                            info_count text,
                            owasp text,
                            tech_list text,
                            config_id text
                            );"""
    app_conn.execute('pragma journal_mode=wal;')

    create_table(app_conn, create_apps)


def create_compliance_table():
    database = r"navi.db"
    compliance_conn = new_db_connection(database)
    create_compliance = """CREATE TABLE IF NOT EXISTS compliance (
                            asset_uuid text,
                            actual_value text,
                            audit_file text,
                            check_id text,
                            check_info text,
                            check_name text,
                            expected_value text,
                            first_seen text,
                            last_seen text,
                            plugin_id text,
                            reference text,
                            see_also text,
                            solution text,
                            status text 
                            );"""

    create_table(compliance_conn, create_compliance)



import click
from .th_asset_export import asset_export
from .th_vuln_export import vuln_export
from .was_data_export import grab_scans
from .th_compliance_export import compliance_export


def threads_check(threads):
    if threads != 10:  # Limit the amount of threads to avoid issues
        click.echo("\nUsing {} thread(s) at your request".format(threads))
        if threads not in range(1, 11):
            click.echo("Enter a value between 1 and 10")
            exit()


@click.group(help="Update the local Navi repository - saved in your current dir")
def update():
    pass


@update.command(help="Perform a full update (30d Vulns / 90d Assets); Delete the current Database")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def full(threads):

    if threads:
        threads_check(threads)

    exid = '0'

    vuln_export(30, exid, threads)
    asset_export(90, exid, threads)


@update.command(help="Update the Asset Table")
@click.option('--days', default='90', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def assets(threads, days, exid):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    asset_export(days, exid, threads)


@update.command(help="Update the vulns Table")
@click.option('--days', default='30', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def vulns(threads, days, exid):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    vuln_export(days, exid, threads)


@update.command(help="Update the Was Data")
def was():
    grab_scans()


@update.command(help="Update the Compliance data")
@click.option('--days', default='30', help="Limit the download to X # of days")
@click.option('--exid', default='0', help="Download using a specified Export ID")
@click.option('--threads', default=10, help="Control the threads to speed up or slow down downloads - (1-10)")
def compliance(threads, days, exid):
    if threads:
        threads_check(threads)

    if exid == ' ':
        exid = '0'

    compliance_export(days, exid, threads)



import click
from .database import new_db_connection
from .api_wrapper import request_data, tenb_connection

tio = tenb_connection()


def check_agroup_exists(aname):
    rvalue = 'no'
    for group in tio.access_groups.list():
        if str(group['name']).lower() == str(aname).lower():
            rvalue = group['id']
    return rvalue


@click.command(help="Create an Access group Based on a Tag")
@click.option('--name', default='', required=True, help="Create an Access group with the following Name")
@click.option('--c', default='', required=True, help="Category name to use: requires --v and Value Name")
@click.option('--v', default='', required=True, help="Tag Value to use; requires --c and Category Name")
@click.option('--user', default='', help="User you want to Assign to the Access Group")
@click.option('--usergroup', default='', help="User Group you want to assign to the Access Group")
@click.option('-scan', is_flag=True, help="Set Scan ONLY permission")
@click.option('-view', is_flag=True, help="Set View ONLY permission")
@click.option('-scanview', is_flag=True, help="Set Scan AND View permissions")
def agroup(name, c, v, user, usergroup, scan, view, scanview):
    new_list = []
    permission = []
    choice = 'none'
    permtype = 'none'

    if user == '' and usergroup == '':
        click.echo("You Need to use '--user' or '--usergroup' command and supply a user or group. e.g: user@yourdomain or Linux Admins")
        exit()

    if user != '':
        permtype = 'user'
        choice = user

    if usergroup != '':
        permtype = 'group'
        choice = usergroup

    if name == '':
        click.echo("You need to use the --name command to name your Access Group")
        exit()

    if c == '':
        click.echo("Tag Category is required.  Please use the --c command")
        exit()

    if v == '':
        click.echo("Tag Value is required. Please use the --v command")
        exit()

    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        cur = conn.cursor()
        cur.execute("SELECT tag_uuid from tags where tag_key='" + c + "' and tag_value='" + v + "';")
        assets = cur.fetchall()

        tag_uuid = assets[0]

        print(tag_uuid)

        if scanview:
            permission = ["CAN_VIEW", "CAN_SCAN"]

        if view:
            permission = ["CAN_VIEW"]

        if scan:
            permission = ["CAN_SCAN"]

        if not scan and not view and not scanview:
            click.echo("\nYou must supply a permission")
            click.echo("\nUse one of the following:\n-scan\n-view\n-scanview\n")
            exit()

        payload = {"name": str(name), "access_group_type": "MANAGE_ASSETS", "rules": [{"type": "tag_uuid", "operator": "set-has", "terms": tag_uuid}],
                   "principals": [{"permissions": permission, "type": permtype, "principal_name": choice}]}

        answer = check_agroup_exists(str(name))

        try:
            # Check to see if the list has any IPs in it.
            if answer == 'no':
                new_access_group = request_data('POST', '/v2/access-groups', payload=payload)
                click.echo("\nYour Access group {} is being created now \n".format(new_access_group['name']))
                click.echo("The UUID is {} \n".format(new_access_group['id']))
            else:
                update_group = request_data('PUT', '/v2/access-groups/' + str(answer), payload=payload)
                click.echo("\nYour Access group {} is being updated now \n".format(update_group['name']))
                click.echo("The UUID is {} \n".format(update_group['id']))
        except TypeError as E:
            click.echo("\nAccess group? - Check the Username")
            click.echo(E)



import boto3
import click
from .database import db_query
from .tag import tag_by_uuid


def organize_aws_keys(aws_ec2):
    aws_tags = {}  # Dictionary of Tag key/Value pairs
    aws_keys = {}  # Dictionary of Values/List of Instance Ids
    for tags in aws_ec2['Tags']:
        if tags['ResourceType'] == 'instance':
            aws_key = tags['Key']
            aws_value = tags['Value']
            resource_id = tags['ResourceId']

            # Tenable IO Requires a Key and a Value; AWS does not.  In this case we just use the key as both
            if aws_value == '':
                aws_value = aws_key

            # Creates a new value dict if one doesn't exist, adds to the value list if one does.
            try:
                aws_keys[aws_value].append(resource_id)
            except KeyError:
                aws_keys[aws_value] = [resource_id]

            # creates a key dict if one doesn't exist, adds the value dict if one does
            if aws_key not in aws_tags:
                aws_tags[aws_key] = {aws_value: aws_keys[aws_value]}
            else:
                aws_tags[aws_key].update({aws_value: aws_keys[aws_value]})

    return aws_tags


@click.command(help="Migrate AWS Tags to T.io tags by Instance ID")
@click.option("--region", default="", required=True, help="AWS Region")
@click.option("--a", default="", required=True, help="Access Key")
@click.option("--s", default="", required=True, help="Secret Key")
def migrate(region, a, s):
    if not a or not s or not region:
        click.echo("You need a region, access key and secret Key")
        exit()
    # Authentication
    ec2client = boto3.client('ec2', region_name=region, aws_access_key_id=a, aws_secret_access_key=s)

    # Grab All of the tags in the Account
    aws_ec2 = ec2client.describe_tags()

    # Send the data to get organized into a neat dictionary of Lists
    aws_organized_tags = organize_aws_keys(aws_ec2)

    # Grab the Key, value and the new list out of the dict to send to the tagging function
    for key, value in aws_organized_tags.items():
        for z, w in value.items():
            uuid_list = []
            for instance in w:
                # Look up the UUID of the asset
                db = db_query("select uuid from assets where aws_id='{}';".format(instance))
                for record in db:
                    uuid_list.append(record[0])
            description = "AWS Tag by Navi"

            print("Creating a Tag named - {} : {} - with the following ids {}".format(key, z, w))

            tag_by_uuid(uuid_list, key, z, description)



import click
from .api_wrapper import tenb_connection
import textwrap

tio = tenb_connection()


@click.group(help="Perform common tasks against Agents and Agent Groups")
def agent():
    pass


@agent.command(help="Display Agent information - Paired with Group IDs")
def display():
    try:
        click.echo("\n{:45s} {:15} {:40} {}".format("Agent Name", "Agent ID", "UUID", "Status", "Group(id)s"))
        click.echo("-" * 150)

        for agent_info in tio.agents.list():
            groups_string = ''
            try:
                for group in agent_info['groups']:
                    groups_string = groups_string + ", {}({})".format(group['name'], group['id'])
            except KeyError:
                pass
            click.echo("{:45s} {:15} {:40s} {}".format(textwrap.shorten(str(agent_info['name']), width=45),
                                                       str(agent_info['id']),
                                                       str(agent_info['uuid']), str(agent_info['status']),
                                                       textwrap.shorten(groups_string[1:], width=60)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@agent.command(help="Display Agent Groups")
@click.option("--gid", default=None, help="Display the agents that are members of the group using the group ID")
def groups(gid):
    if gid:
        group_details = tio.agent_groups.details(gid)

        click.echo("\n{:85s} {:15} {:40}".format("Agent Name", "Agent ID", "UUID", "Status"))
        click.echo("-" * 150)

        for agent_info in group_details['agents']:
            click.echo("{:85s} {:15} {:40s}".format(textwrap.shorten(str(agent_info['name']), width=85),
                                                    str(agent_info['id']),
                                                    str(agent_info['uuid']), str(agent_info['status'])))

        click.echo()
    else:

        try:
            click.echo("\n{:45} {:40} {:10}".format("Group Name", "Group UUID", "Group ID"))
            click.echo("-" * 150)

            group_list = tio.agent_groups.list()

            for group in group_list:
                click.echo("{:45} {:40} {:10}".format(str(group['name']), str(group['uuid']), str(group['id'])))

            click.echo()
        except AttributeError:
            click.echo("\nCheck your permissions or your API keys\n")


@agent.command(help="Create a new Agent Group")
@click.option("--name", default=None, help="The name of the new Agent Group you want to create")
@click.option("--scanner", default=1, help="Add Agent Group to a specific scanner")
def create(name, scanner):
    group_creation = tio.agent_groups.create(name=name, scanner_id=scanner)

    click.echo("\nYour agent group: {} has been created.\n\nHere is the ID: {} and UUID : {}"
               "\n".format(group_creation['name'], str(group_creation['id']), str(group_creation['uuid'])))


@agent.command(help="Add an agent to a Group")
@click.option("--aid", default=None, help="The agent ID of the agent you want to add ")
@click.option("--gid", default=None, help="The Group ID you want to add the agent(s) to.")
def add(aid, gid):
    if gid:
        if aid:
            # Add agent to Group
            tio.agent_groups.add_agent(gid, aid)

            click.echo("\nYour agent has been added to the Group ID: {}".format(gid))


@agent.command(help="Remove an Agent from a Agent Group")
@click.option("--aid", default=None, help="The agent ID of the agent you want to remove ")
@click.option("--gid", default=None, help="The Group ID you want to add the agent(s) to.")
def remove(aid, gid):
    if gid:
        if aid:
            # Remove an agent from a Group
            tio.agent_groups.delete_agent(gid, aid)

            click.echo("\nYour agent has been removed from the Group ID: {}".format(gid))


@agent.command(help="Unlink an by Agent ID")
@click.option("--aid", default=None, help="The Agent ID of the agent you want to unlink")
def unlink(aid):
    if aid:
        tio.agents.unlink(aid)
        click.echo("\nYour Agent: {} has been unlinked".format(aid))
    else:
        click.echo("You have provide a agent ID")



import click
import time
import arrow
from .api_wrapper import tenb_connection, navi_version, request_data
from .scanners import nessus_scanners
from .database import new_db_connection, db_query
from .licensed_count import get_licensed
from sqlite3 import Error
import textwrap

tio = tenb_connection()


@click.group(help="Display or Print information found in Tenable.io")
def display():
    pass


@display.command(help="List all of the scanners")
def scanners():
    nessus_scanners()


@display.command(help="List all of the Users")
def users():
    try:
        click.echo("\n{:34s} {:40s} {:40s} {:10s} {}".format("User Name", "Login Email", "UUID", "ID", "Enabled"))
        click.echo("-" * 150)
        for user in tio.users.list():
            click.echo("{:34s} {:40s} {:40s} {:10s} {}".format(str(user["user_name"]), str(user["username"]), str(user['uuid']), str(user['id']), str(user['enabled'])))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List all Exclusions")
def exclusions():
    try:
        for exclusion in tio.exclusions.list():
            click.echo("\n{} {}".format("Exclusion Name :", exclusion["name"]))
            click.echo("-" * 150)
            click.echo("{}".format(str(exclusion["members"])))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List all containers and their Vulnerability  Scores")
def containers():
    try:
        # Use CS module
        resp = tio.get('container-security/api/v2/images?limit=1000')
        data = resp.json()

        click.echo("{:35s} {:35s} {:15s} {:15s} {:10s}".format("Container Name", "Repository ID", "Tag", "Docker ID", "# of Vulns"))
        click.echo("-" * 150)

        try:
            for images in data["items"]:
                click.echo("{:35s} {:35s} {:15s} {:15s} {:10s}".format(images["name"], images["repoName"],
                                                                       images["tag"], str(images["imageHash"]),
                                                                       str(images["numberOfVulns"])))
        except KeyError:
            pass

        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List The actor and the action in the log file")
def logs():
    try:
        events = tio.audit_log.events()
        click.echo("{:24s} {:30s} {}".format("Event Date", "Action Taken", "User"))
        click.echo("-" * 150)

        for log in events:
            click.echo("{:24s} {:30s} {:30s}".format(str(log['received']), str(log['action']),
                                                     str(log['actor']['name'])))

        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List the running Scans")
def running():
    try:
        click.echo("\n{:60s} {:10s} {:30s}".format("Scan Name", "Scan ID", "Status"))
        click.echo("-" * 150)

        for scan in tio.scans.list():
            if scan['status'] == "running":
                click.echo("{:60s} {:10s} {:30s}".format(str(scan['name']), str(scan['id']), str(scan['status'])))

        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List all Scans")
def scans():
    try:
        click.echo("\n{:60s} {:10s} {:30s} {}".format("Scan Name", "Scan ID", "Status", "UUID"))
        click.echo("-" * 150)

        for scan in tio.scans.list():
            click.echo("{:60s} {:10s} {:30s} {}".format(str(scan['name']), str(scan['id']), str(scan['status']),
                                                        str(scan['uuid'])))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Nessus Network Monitor assets and their vulnerability scores")
def nnm():
    click.echo("\nChecking all NNM scanners for assets recently found...")
    click.echo("\n{:20} {:10} {}".format("IP Address", "Score", "Scan ID"))
    click.echo("-" * 150)

    for scan in tio.scans.list():
        try:
            if str(scan["type"]) == "pvs":
                resp = tio.get('scans/{}'.format(scan["id"]))
                data = resp.json()

                for host in data["hosts"]:
                    click.echo("{:20} {:10} {}".format(str(host["hostname"]), str(host["score"]), str(scan["id"])))

        except AttributeError:
            click.echo("\nCheck your permissions or your API keys\n")

        except KeyError:
            pass


@display.command(help="Assets found in the last 30 days")
@click.option("--tag", default='', help="Assets that are apart of a given Tag.  Use Value UUID")
def assets(tag):
    if tag:
        data = db_query("select ip_address, fqdn, aes, acr from assets LEFT JOIN tags ON uuid == asset_uuid where tag_uuid=='{}';".format(tag))

        click.echo("\nBelow are the assets that are apart of the Tag")
        click.echo("\n{:36} {:65} {:6} {}".format("IP Address", "FQDN", "AES", "ACR"))
        click.echo("-" * 150)
        try:
            for asset in data:
                ipv4 = str(asset[0])
                fqdn = str(asset[1])
                exposure_score = str(asset[2])
                acr = str(asset[3])

                click.echo("{:36} {:65} {:6} {}".format(ipv4, fqdn, exposure_score, acr))
            click.echo()
        except TypeError:
            click.echo("\nThe Tag has no assets or the tag hasn't finished being processed by T.io\n")
    else:
        try:
            click.echo("\nBelow are the assets found in the last 30 days")
            click.echo("\n{:16} {:65} {:40} {:6} {}".format("IP Address", "FQDN", "UUID", "AES", "Sources"))
            click.echo("-" * 150)

            count = 0
            for asset in tio.workbenches.assets():
                count += 1
                sources = ""
                try:
                    exposure_score = str(asset["exposure_score"])
                except KeyError:
                    exposure_score = "- -"

                try:
                    uuid = asset["id"]
                except IndexError:
                    uuid = " - - - - - - - "
                try:
                    fqdn = asset["fqdn"][0]
                except IndexError:
                    fqdn = " - - - - - - - "

                try:
                    ipv4 = asset["ipv4"][0]
                except IndexError:
                    ipv4 = " - - - - - - - "

                for source in asset["sources"]:
                    sources += ", {}".format(str(source['name']))

                click.echo("{:16} {:65} {:40} {:6} {}".format(ipv4, fqdn, uuid, exposure_score, sources[1:]))

            click.echo("\nTotal: {}".format(count))
        except AttributeError:
            click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List Scan Policies")
def policies():
    try:
        click.echo("\n{:40s} {:51s} {:10} {}".format("Policy Name", "Description", "ID", "Template ID"))
        click.echo("-" * 150)

        for policy in tio.policies.list():
            click.echo("{:40s} {:51s} {:10} {}".format(str(policy['name']), str(policy['description']), str(policy['id']),
                                                       policy['template_uuid']))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List Connector Details and Status")
def connectors():
    try:
        resp = tio.get('settings/connectors')
        data = resp.json()

        click.echo("\n{:11s} {:40s} {:40s} {:30s} {}".format("Type", "Connector Name", "Connector ID", "Last Sync", "Schedule"))
        click.echo("-" * 150)

        for conn in data["connectors"]:
            schedule = str(conn['schedule']['value']) + " " + str(conn['schedule']['units'])

            try:
                last_sync = conn['last_sync_time']
            except KeyError:
                last_sync = "Hasn't synced"

            click.echo("{:11s} {:40s} {:40s} {:30s} {}".format(str(conn['type']), str(conn['name']), str(conn['id']),
                                                               last_sync, schedule))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List Access Groups")
def agroups():
    rules = "Not Rule Based"
    try:
        click.echo("\n{:25s} {:40s} {:25} {}".format("Group Name", "Group ID", "Last Updated", "Rules"))
        click.echo("-" * 150)

        for group in tio.access_groups.list():

            try:
                updated = group['updated_at']
            except KeyError:
                updated = "Not Updated"

            details = tio.access_groups.details(group['id'])

            try:
                for rule in details['rules']:
                    rules = str(rule['terms'])
            except KeyError:
                rules = "Not Rule Based"

            click.echo("{:25s} {:40s} {:25} {:60s}".format(str(group['name']), str(group['id']), str(updated),
                                                           textwrap.shorten(rules, width=60)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print T.io Status and Account info")
def status():
    try:
        data = tio.server.properties()
        session_data = tio.session.details()
        click.echo("\nTenable IO Information")
        click.echo("-" * 25)
        click.echo("{} {}".format("Container ID : ", session_data["container_id"]))
        click.echo("{} {}".format("Container UUID :", session_data["container_uuid"]))
        click.echo("{} {}".format("Container Name : ", session_data["container_name"]))
        click.echo("{} {}".format("Site ID :", data["analytics"]["site_id"]))
        click.echo("{} {}".format("Region : ", data["region"]))

        click.echo("\nLicense information")
        click.echo("-" * 25)
        click.echo("{} {}".format("Licensed Assets : ", get_licensed()))
        click.echo("{} {}".format("Agents Used : ", data['license']["agents"]))
        try:
            click.echo("{} {}".format("Expiration Date : ", data['license']["expiration_date"]))
        except KeyError:
            pass
        click.echo("{} {}".format("Scanners Used : ", data['license']["scanners"]))
        click.echo("{} {}".format("Users : ", data["license"]["users"]))

        click.echo("\nEnabled Apps")
        click.echo("-" * 15)
        click.echo()
        try:
            for key in data["license"]["apps"]:
                click.echo(key)
                click.echo("-" * 5)
                try:
                    click.echo("{} {}".format("Expiration: ", str(data["license"]["apps"][key]["expiration_date"])))
                except KeyError:
                    pass
                click.echo("{} {}".format("Mode: ", str(data["license"]["apps"][key]["mode"])))
                click.echo()
        except KeyError:
            pass

    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Agent information")
def agents():
    try:
        click.echo("\n{:30s} {:20} {:20} {:20} {:6} {}".format("Agent Name", "IP Address", "Last Connect Time", "Last Scanned Time", "Status", "Group(id)s"))
        click.echo("-" * 150)

        for agent in tio.agents.list():
            try:
                last_connect = agent['last_connect']
                last_connect_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_connect))
            except KeyError:
                last_connect_time = "Unknown"

            try:
                last_scanned = agent['last_scanned']
                last_scanned_time = time.strftime("%b %d %H:%M:%S", time.localtime(last_scanned))
            except KeyError:
                # I assume if we can't pull as scanned time, it doesn't exist
                last_scanned_time = "Not Scanned"
            groups = ''
            try:
                for group in agent['groups']:
                    groups = groups + ", {}({})".format(group['name'], group['id'])
            except KeyError:
                pass
            click.echo("{:30s} {:20s} {:20s} {:20s} {:6s} {}".format(str(agent['name']), str(agent['ip']),
                                                                     str(last_connect_time), str(last_scanned_time),
                                                                     str(agent['status']), textwrap.shorten(groups[1:], width=60)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Target Groups")
def tgroups():
    try:
        print("\nTarget Group Name".ljust(41), "TG ID".ljust(10), "Owner".ljust(30), "Members")
        print("-" * 100)
        for targets in tio.target_groups.list():
            mem = targets['members']
            print(str(targets['name']).ljust(40), str(targets['id']).ljust(10), str(targets['owner']).ljust(30), textwrap.shorten(mem, width=60))
        print()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Licensed Assets")
def licensed():
    try:
        click.echo("\n{} {}".format("Licensed Count: ", get_licensed()))
        click.echo()
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT ip_address, fqdn, last_licensed_scan_date from assets where last_licensed_scan_date !=' ';")
            data = cur.fetchall()

            click.echo("{:20s} {:65s} {}".format("IP Address", "Full Qualified Domain Name", "Licensed Date"))
            click.echo("-" * 150)
            click.echo()
            count = 0
            for asset in data:
                count += 1
                ipv4 = asset[0]
                fqdn = asset[1]
                licensed_date = asset[2]
                # Don't display Web applications in this output
                if ipv4 != " ":
                    click.echo("{:20s} {:65s} {}".format(str(ipv4), str(fqdn), licensed_date))
        click.echo("\nTotal: {}".format(count))
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Tag Information")
def tags():
    try:
        click.echo("\n{:30s} {:35s} {}".format("Category", "  Value", "  Value UUID"))
        click.echo("-" * 150)
        for tag_values in tio.tags.list():
            try:
                tag_value = tag_values['value']
                uuid = tag_values['uuid']
            except KeyError:
                tag_value = "Value Not Set Yet"
                uuid = "NO Value set"
            click.echo("{:30s} : {:35s} {}".format(str(tag_values['category_name']), str(tag_value), str(uuid)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Tag Categories")
def categories():
    try:
        click.echo("\n{:31s} {}".format("Tag Categories", "Category UUID"))
        click.echo("-" * 150)
        for cats in tio.tags.list_categories():
            category_name = cats['name']
            category_uuid = cats['uuid']
            click.echo("{:31s} {}".format(str(category_name), str(category_uuid)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print SMTP information")
def smtp():
    try:
        database = r"navi.db"
        conn = new_db_connection(database)
        with conn:
            cur = conn.cursor()
            cur.execute("SELECT server, port, from_email from smtp;")
            data = cur.fetchall()
            for settings in data:
                click.echo("\nYour email server: {}".format(settings[0]))
                click.echo("The email port is: {}".format(settings[1]))
                click.echo("Your email is: {}\n".format(settings[2]))
    except Error:
        click.echo("\nYou have no SMTP information saved.\n")


@display.command(help="Print Cloud information")
def cloud():
    try:
        click.echo("\n{:13s} {:15s} {:50s} {:40} {}".format("Source", "IP", "FQDN", "UUID", "First seen"))
        click.echo("-" * 150)

        for cloud_assets in tio.workbenches.assets(('sources', 'set-has', 'AWS'), ('sources', 'set-has', 'GCP'),
                                                   ('sources', 'set-has', 'AZURE'), filter_type="or", age=90):
            for source in cloud_assets['sources']:
                if source['name'] != 'NESSUS_SCAN':

                    try:
                        asset_ip = cloud_assets['ipv4'][0]
                    except IndexError:
                        asset_ip = "No Ip found"

                    uuid = cloud_assets['id']

                    try:
                        asset_fqdn = cloud_assets['fqdn'][0]
                    except IndexError:
                        asset_fqdn = "NO FQDN found"

                    click.echo("{:13s} {:15s} {:50s} {:40s} {}".format(str(source['name']), str(asset_ip),
                                                                       str(asset_fqdn), str(uuid),
                                                                       str(source['first_seen'])))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Network Information")
def networks():
    try:
        click.echo("\n{:45s} {:16} {}".format("Network Name", "# of Scanners", "UUID"))
        click.echo("-" * 150)

        for network in tio.networks.list():
            click.echo("{:45s} {:16} {}".format(str(network['name']), str(network['scanner_count']),
                                                str(network['uuid'])))
        click.echo()

    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Navi Version")
def version():
    click.echo("\nCurrent Navi Version: {}\n".format(navi_version()))


@display.command(help="Print User group information")
@click.option('--membership', required=True, default='', help="Display Users that apart of a particular usergroup")
def usergroups(membership):
    try:
        if membership:
            click.echo("\n{:35s} {:40s} {:40s} {:10} {}".format("User Name", "Login email", "User UUID", "User ID",
                                                                "Enabled?"))
            click.echo("-" * 150)
            for user in tio.groups.list_users(membership):
                click.echo("{:35s} {:40s} {:40s} {:10} {}".format(str(user["name"]), str(user["username"]),
                                                                  str(user['uuid']), str(user['id']),
                                                                  str(user["enabled"])))
            click.echo()
        else:
            click.echo("\n{:35s} {:10s} {:40s} {}".format("Group Name", "Group ID", "Group UUID", "User Count"))
            click.echo("-" * 150)

            for user_group in tio.groups.list():
                click.echo("{:35s} {:10s} {:40s} {}".format(str(user_group['name']), str(user_group['id']),
                                                            str(user_group['uuid']), str(user_group['user_count'])))
            click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="List All Credentials")
def credentials():
    try:

        click.echo("\n{:25s} {:25s} {:25s} {:25s} {:40s}".format("Credential Name", "Created By", "Credential Type", "Category", "Credential UUID"))
        click.echo("-" * 150)

        for cred in tio.credentials.list():
            creator = cred['created_by']['display_name']
            name = cred['name']
            cred_type = cred['type']['name']
            cred_uuid = cred['uuid']
            category = cred['category']['name']
            click.echo("{:25s} {:25s} {:25s} {:25s} {:40s}".format(textwrap.shorten(name, width=25),
                                                                   textwrap.shorten(creator, width=25),
                                                                   textwrap.shorten(cred_type, width=25),
                                                                   textwrap.shorten(category, width=25),
                                                                   textwrap.shorten(cred_uuid, width=40)))
        click.echo()
    except AttributeError:
        click.echo("\nCheck your permissions or your API keys\n")


@display.command(help="Print Asset and Vulnerability Export Job information")
@click.option('-a', help="Display Asset Export Jobs", is_flag=True, required=True)
@click.option('-v', help="Display Vulnerability Export Jobs", is_flag=True, required=True)
def exports(a, v):
    if not a and not v:
        click.echo("\nYou need to use '-a' or '-v' to select your export type. (assets vs vulns)\n")

    current_time = time.time()
    time_frame = (current_time - (86400 * 3)) * 1000
    if a:
        export_data = request_data('GET', '/assets/export/status')
        click.echo("{:45s} {:20s} {:10s} {:45s} {:10s} {}".format("\nAsset Export UUID", "Created Date", "Status",
                                                                  "Export Filter Used",  "Chunk Size", "Total Chunks"))
        click.echo('-' * 150)

        for export in export_data['exports']:
            compare_time = export['created']
            newtime = arrow.Arrow.fromtimestamp(compare_time)

            if compare_time > time_frame:
                export_uuid = export['uuid']
                export_status = export['status']
                export_chunk_size = export['num_assets_per_chunk']
                export_filter = export['filters']
                export_total_chunks = export['total_chunks']

                click.echo("{:44s} {:20s} {:10s} {:45s} {:10d} {}".format(export_uuid, newtime.format('MM-DD-YYYY'),
                                                                          export_status, export_filter,
                                                                          export_chunk_size, export_total_chunks))

    if v:
        vuln_export_data = request_data('GET', '/vulns/export/status')
        click.echo("{:45s} {:20s} {:10s} {:45s} {:10s} {}".format("\nVulnerability Export UUID", "Created Date",
                                                                  "Status", "States", "Chunk Size", "Total Chunks"))
        click.echo('-' * 150)

        for vuln_export in vuln_export_data['exports']:
            vuln_compare_time = vuln_export['created']
            vuln_newtime = arrow.Arrow.fromtimestamp(vuln_compare_time)

            if vuln_compare_time > time_frame:
                export_uuid = vuln_export['uuid']
                export_status = vuln_export['status']
                export_chunk_size = vuln_export['num_assets_per_chunk']
                export_filter = str(vuln_export['filters']['state'])
                vuln_export_total_chunks = vuln_export['total_chunks']

                click.echo("{:44s} {:20s} {:10s} {:45s} {:10d} {}".format(export_uuid, vuln_newtime.format('MM-DD-YYYY'),
                                                                          export_status, export_filter,
                                                                          export_chunk_size, vuln_export_total_chunks))

    click.echo()


@display.command(help="Print Auth information for each user")
@click.argument('uid')
def auth(uid):
    info = request_data("GET", "/users/{}/authorizations".format(uid))

    click.echo("\n{:45} {:20} {:20} {:20} {}".format("Account_UUID", "API Permitted", "Password Permitted",
                                                     "SAML Permitted", "User_UUID"))
    click.echo("-" * 150)

    click.echo("{:45} {:20} {:20} {:20} {}".format(str(info['account_uuid']), str(info['api_permitted']),
                                                   str(info['password_permitted']), str(info['saml_permitted']),
                                                   str(info['user_uuid'])))

    click.echo()


@display.command(help="List Scan Policies")
@click.option("-policy", is_flag=True, help="Display all Policy Templates")
@click.option("-scan", is_flag=True, help="Display all Scan Templates")
def templates(policy, scan):
    template_type = ""

    if policy:
        template_type = "policy"
    if scan:
        template_type = "scan"

    if template_type:
        try:
            click.echo("\n{:40s} {:61s} {}".format("Policy Name", "Description", "Template ID"))
            click.echo("-" * 150)

            for policy in tio.editor.template_list(str(template_type)):
                click.echo("{:40s} {:61s} {}".format(str(policy['name']), str(policy['title']),
                                                     policy['uuid']))
            click.echo()
        except AttributeError:
            click.echo("\nCheck your permissions or your API keys\n")
    else:
        click.echo("\nYou must use '-scan' or '-policy'")


@display.command(help="List Completed Audit files and Audit information")
@click.option('--name', default=None, help="List all of the Assets with completed Audits for the Given Audit name")
@click.option('--uuid', default=None, help="List all compliance findings for a given Asset UUID")
def audits(name, uuid):

    if name and uuid:
        data = db_query("SELECT assets.fqdn, compliance.check_name, compliance.status FROM compliance LEFT OUTER JOIN assets ON "
                        "assets.uuid = compliance.asset_uuid "
                        "where compliance.audit_file='{}' and compliance.asset_uuid='{}';".format(name, uuid))

        click.echo("{:65} {:65} {}".format("\nFQDN", " Check Name", " Status"))
        click.echo("-" * 150)
        click.echo()
        for finding in data:
            click.echo("{:65} {:65} {}".format(textwrap.shorten(str(finding[0]), width=65),
                                               textwrap.shorten(str(finding[1]), width=65),
                                               finding[2]))
        click.echo()

    elif name:
        data = db_query("SELECT assets.fqdn, compliance.check_name, compliance.status FROM compliance LEFT OUTER JOIN assets ON "
                        "assets.uuid = compliance.asset_uuid "
                        "where audit_file='{}';".format(name))

        click.echo("{:65} {:65} {}".format("\nFQDN", " Check Name", " Status"))
        click.echo("-" * 150)
        click.echo()

        for finding in data:
            click.echo("{:65} {:65} {}".format(textwrap.shorten(str(finding[0]), width=65),
                                               textwrap.shorten(str(finding[1]), width=65),
                                               finding[2]))
        click.echo()

    elif uuid:
        data = db_query("SELECT assets.fqdn, compliance.check_name, compliance.status FROM compliance LEFT OUTER JOIN assets ON "
                        "assets.uuid = compliance.asset_uuid "
                        "where asset_uuid='{}';".format(uuid))

        click.echo("{:65} {:65} {}".format("\nFQDN", " Check Name", " Status"))
        click.echo("-" * 150)
        click.echo()
        for finding in data:
            click.echo("{:65} {:65} {}".format(textwrap.shorten(str(finding[0]), width=65),
                                               textwrap.shorten(str(finding[1]), width=65),
                                               finding[2]))
        click.echo()

    else:
        compliance_data = db_query("SELECT audit_file from compliance;")
        compliance_list = []

        for audit in compliance_data:
            if audit not in compliance_list:
                compliance_list.append(audit)

        click.echo("\nCompleted Audits")
        click.echo("-" * 80)
        click.echo()

        for name in compliance_list:
            click.echo(name[0])

        click.echo()



import click
from .api_wrapper import request_data
from IPy import IP


def find_target_group(tg_name):

    data = request_data("GET", '/target-groups')
    group_id = 0
    for group in data['target_groups']:
        try:
            if group['name'] == tg_name:
                group_id = group['id']
        except:
            pass
    return group_id


def create_target_group(target_name, tg_list):

    # Check to see if the Target group exists
    group_id = find_target_group(target_name)

    # Target group API takes a string of IPs. We will start the string here.
    trgstring = ""

    # Check to see if tg_list is a string
    string_test = isinstance(tg_list, str)

    # turn the list into a string separated by a comma
    if not string_test:
        for ips in tg_list:
            trgstring = trgstring + str(ips) + ","
    else:
        trgstring = tg_list

    if not tg_list:
        click.echo("\nYour request returned zero results\nAs a result, nothing happened\n")
        exit()

    click.echo("\nThese are the IPs that will be added to the target Group: {}".format(target_name))
    click.echo(tg_list)
    click.echo()

    if group_id != 0:
        # Update current Target Group
        payload = {"name": target_name, "members": trgstring, "type": "system"}
        request_data("PUT", '/target-groups/'+str(group_id), payload=payload)
    else:
        # Create a New Target Group
        payload = {"name": target_name, "members": str(trgstring), "type": "system", "acls": [{"type": "default", "permissions": 64}]}
        request_data("POST", '/target-groups', payload=payload)


def cloud_to_target_group(cloud, days, choice, target_group_name):
    query = {"date_range": days, "filter.0.filter": "sources", "filter.0.quality": "set-has", "filter.0.value": cloud}
    data = request_data('GET', '/workbenches/assets', params=query)
    target_ips = []

    for assets in data['assets']:
        target_ip_list = assets['ipv4']
        # loop through all of the IPs
        for ip in target_ip_list:
            # Check to IP type
            check_ip = IP(ip)
            check = check_ip.iptype()
            if check == choice:
                # Add the IP if there is a match
                target_ips.append(ip)

    create_target_group(target_group_name, target_ips)


@click.command(help="Create a Target Group")
@click.option('--name', default='', required=True, help="Target Group Name")
@click.option('--ip', default='', help="Ip(s) or subnet(s) separated by coma")
@click.option('-aws', is_flag=True, help="Turn AWS assets found by the connector into a Target Group")
@click.option('-gcp', is_flag=True, help="Turn GCP assets found by the connector into a Target Group")
@click.option('-azure', is_flag=True, help="Turn Azure assets found by the connector into a Target Group")
@click.option('--days', default='30', help="Set the number of days for the IPs found by the connector. Requires: aws, gcp, or azure")
@click.option('-priv', is_flag=True, help="Set the IP(s) to be used as Private")
@click.option('-pub', is_flag=True, help="Set the IP to be used as Public")
@click.option('-migrate', is_flag=True, help="Migrate Target Groups to Tags")
def tgroup(name, ip, aws, gcp, azure, days, priv, pub, migrate):
    choice = 'PUBLIC'

    if priv:
        choice = 'PRIVATE'

    if pub:
        choice = 'PUBLIC'

    if ip != '':
        create_target_group(name, ip)

    if aws:
        cloud_to_target_group("AWS", days, choice, name)

    if gcp:
        cloud_to_target_group("GCP", days, choice, name)

    if azure:
        cloud_to_target_group("AZURE", days, choice, name)

    if migrate:

        tgroups = request_data('GET', '/target-groups')

        for group in tgroups['target_groups']:
            member = group['members']
            name = group['name']
            group_type = group['type']
            d = "Imported by Script"
            try:
                if name != 'Default':
                    payload = {"category_name": str(group_type), "value": str(name), "description": str(d), "filters": {"asset": {"and": [{"field": "ipv4", "operator": "eq", "value": str(member)}]}}}
                    data = request_data('POST', '/tags/values', payload=payload)

                    value_uuid = data["uuid"]
                    cat_uuid = data['category_uuid']
                    print("\nI've created your new Tag - {} : {}\n".format(group_type, name))
                    print("The Category UUID is : {}\n".format(cat_uuid))
                    print("The Value UUID is : {}\n".format(value_uuid))
            except:
                pass



import csv
from sqlite3 import Error
from .database import new_db_connection


def csv_export():
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:

        # Create our headers - We will Add these two our list in order
        header_list = ["IP Address", "Hostname", "FQDN", "UUID", "First Found", "Last Found", "Operating System",
                       "Mac Address", "Agent-UUID", "last Licensed Scan Date", "Network ID", "ACR Score", "AES Score",
                       "AWS ID"]

        cur = conn.cursor()
        try:
            cur.execute("SELECT * from assets;")
        except Error:
            print("\n No data! \n Please run 'navi update' first.\n")
        data = cur.fetchall()

        # Crete a csv file object
        with open('asset_data.csv', mode='w') as csv_file:
            agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')

            # write our Header information first
            agent_writer.writerow(header_list)

            # Loop through each asset
            for assets in data:
                agent_writer.writerow(assets)



import csv
from sqlite3 import Error
from .database import new_db_connection


def licensed_export():
    database = r"navi.db"
    conn = new_db_connection(database)
    with conn:
        with open('licensed_data.csv', mode='w') as csv_file:
            agent_writer = csv.writer(csv_file, delimiter=',', quotechar='"')
            header_list = ["IP Address", "FQDN", "UUID", "Last Licensed Scan Date"]
            agent_writer.writerow(header_list)

            cur = conn.cursor()
            try:
                cur.execute("SELECT ip_address, fqdn, uuid, last_licensed_scan_date from assets where last_licensed_scan_date != ' ';")
            except Error:
                print("\n No data! \n Please run 'navi update' first.\n")
            data = cur.fetchall()

            for asset in data:
                agent_writer.writerow(asset)



import click
import smtplib


def send_email(from_email, to_email, msg, mail_server, password, port):
    click.echo(msg)
    try:
        server = smtplib.SMTP(mail_server, port)
        server.ehlo()
        server.starttls()
        server.login(from_email, password)
        server.sendmail(from_email, to_email, msg)
        server.close()

        click.echo('Email sent!')
    except Exception as E:
        click.echo('\nSomething went wrong...Your email information may be incorrect\n')
        click.echo("For now you must use an 'less secure' setting in Gmail for this feature."
                   "\n I'm working on a more secure option.")
        click.echo(E)
